@@ -1,6 +1,6 @@
 package Jifty::Script;
+use App::CLI;
 use base qw/App::CLI App::CLI::Command/;
-use Jifty::I18N;
 
 =head2 prepare
 
