@@ -14,14 +14,19 @@ Jifty::Web - Web framework for a Jifty application
 use Jifty::Everything;
 use CGI::Cookie;
 use XML::Writer;
-use base qw/Class::Accessor::Fast Jifty::Object/;
+use CSS::Squish;
+use Digest::MD5 qw(md5_hex);
+use base qw/Class::Accessor::Fast Class::Data::Inheritable Jifty::Object/;
 
 use vars qw/$SERIAL/;
 
 __PACKAGE__->mk_accessors(
-    qw(next_page request response session temporary_current_user)
+    qw(next_page request response session temporary_current_user _current_user)
 );
 
+__PACKAGE__->mk_classdata($_)
+    for qw(cached_css cached_css_digest cached_css_mtime);
+
 =head1 METHODS
 
 =head3 new
@@ -146,28 +151,45 @@ in the session.  These are L<Jifty::Record> objects.
 If a temporary_current_user has been set, will return that instead.
 
 If the current application has no loaded current user, we get an empty
-app-specific C<CurrentUser> object. (This is determined by
-theC<framework> configuration varialbe C<CurrentUserClass> and
-defaults to $AppNameC<::CurrentUser>, a subclass of
-L<Jifty::CurrentUser>.  $AppNameC<::CurrentUser> is autogenerated if
-it doesn't exist.
+app-specific C<CurrentUser> object. (This
+C<ApplicationClass>::CurrentUser class, a subclass of
+L<Jifty::CurrentUser>, is autogenerated if it doesn't exist).
 
 =cut
 
 sub current_user {
     my $self = shift;
     if (@_) {
-        my $user = shift;
-         $self->session->set('user' => $user);
+        my $currentuser_obj = shift;
+        $self->session->set(
+            'user_id' => $currentuser_obj ? $currentuser_obj->id : undef );
+        $self->session->set( 'user_ref' => $currentuser_obj
+            ? ref $currentuser_obj->user_object
+            : undef );
+        $self->_current_user( $currentuser_obj || undef );
     }
-    if (defined $self->temporary_current_user) {
+
+    if ( defined $self->temporary_current_user ) {
         return $self->temporary_current_user;
-    } elsif ($self->session->get('user')) {
-        return $self->session->get('user');
-    }
-    else {
-        my $object = Jifty->config->framework('CurrentUserClass')->new();
+    } elsif ( defined $self->_current_user ) {
+        return $self->_current_user;
+
+    } elsif ( my $id = $self->session->get('user_id') ) {
+        my $object = (
+            Jifty->config->framework('ApplicationClass') . "::CurrentUser" )
+            ->new();
+        my $user
+            = $self->session->get('user_ref')->new( current_user => $object );
+        $user->load_by_cols( id => $id );
+        $object->user_object($user);
+        $self->_current_user($object);
+        return $object;
+    } else {
+        my $object = (
+            Jifty->config->framework('ApplicationClass') . "::CurrentUser" )
+            ->new();
         $object->is_superuser(1) if Jifty->config->framework('AdminMode');
+        $self->_current_user($object);
         return ($object);
     }
 }
@@ -218,6 +240,7 @@ sub handle_request {
     for my $request_action ( $self->request->actions ) {
         $self->log->debug("Found action ".$request_action->class . " " . $request_action->moniker);
         next unless $request_action->active;
+        next if $request_action->has_run;
         unless ( Jifty->api->is_allowed( $request_action->class ) ) {
             $self->log->warn( "Attempt to call denied action '"
                     . $request_action->class
@@ -265,9 +288,11 @@ sub handle_request {
             $request_action->has_run(1);
 
             if ( my $err = $@ ) {
-                # poor man's exception propagation
-                # We need to get "LAST RULE" exceptions back up to the dispatcher
-                die $err if ( $err =~ /^LAST RULE/ );
+                # Poor man's exception propagation; we need to get
+                # "LAST RULE" and "ABORT" exceptions back up to the
+                # dispatcher.  This is specifically for redirects from
+                # actions
+                die $err if ( $err =~ /^(LAST RULE|ABORT)/ );
                 $self->log->fatal($err);
                 $action->result->error(
                     Jifty->config->framework("DevelMode")
@@ -283,14 +308,11 @@ sub handle_request {
     }
     $self->session->set_cookie();
 
-    $self->request->call_continuation
-        if $self->response->success;
+    # If there's a continuation call, don't do the rest of this
+    return if $self->response->success and $self->request->call_continuation;
 
     $self->redirect if $self->redirect_required;
     $self->request->do_mapping;
-
-    # This may be a request for fragments, not for a whole page
-    $self->serve_fragments if $self->request->fragments;
 }
 
 =head3 request [VALUE]
@@ -319,10 +341,9 @@ sub form {
 
 Creates a new action (an instance of a subclass of L<Jifty::Action>)
 
-C<CLASS> is appended to the C<ActionBasePath> found in the
-configuration file, and an instance of that class is created, passing
-the C<Jifty::Web> object, the C<MONIKER>, and any other arguments that
-C<new_action> was supplied.
+C<CLASS> is L<qualified|Jifty::Util/qualify>, and an instance of that
+class is created, passing the C<Jifty::Web> object, the C<MONIKER>,
+and any other arguments that C<new_action> was supplied.
 
 C<MONIKER> is a unique designator of an action on a page.  The moniker
 is content-free and non-fattening, and may be auto-generated.  It is
@@ -337,6 +358,11 @@ action with a matching moniker, any arguments that are in that
 requested action but not in the C<PARAMHASH> list are set.  This
 implements "sticky fields".
 
+As a contrast to L<Jifty::Web::Form/add_action>, this does not add the
+action to the current form -- instead, the first form field to be
+rendered will automatically register the action in the current form
+field at that time.
+
 =cut
 
 sub new_action {
@@ -380,7 +406,7 @@ sub new_action {
 
     my $action;
     # XXX TODO bullet proof
-    eval { $action = $class->new( %args, arguments => {%arguments} ); };
+    eval { $action = $class->new( %args, arguments => {%arguments}, current_user => $self->current_user ); };
     if ($@) {
         my $err = $@;
         $self->log->fatal($err);
@@ -469,7 +495,8 @@ sub redirect_required {
 
     if ($self->next_page
         and ( ( $self->next_page ne $self->request->path )
-            or $self->request->state_variables )
+            or $self->request->state_variables
+            or $self->{'state_variables'} )
         )
     {
         return (1);
@@ -492,25 +519,31 @@ sub redirect {
     my $self = shift;
     my $page = shift || $self->next_page;
 
-    my @unrun = grep {not $_->has_run} Jifty->web->request->actions;
+    my @actions = Jifty->web->request->actions;
 
     if (   $self->response->results
         or $self->request->state_variables
-        or @unrun )
+        or $self->{'state_variables'}
+        or $self->request->continuation
+        or @actions )
     {
         my $request = Jifty::Request->new();
         $request->path($page);
         $request->add_state_variable( key => $_->key, value => $_->value )
           for $self->request->state_variables;
-        for (@unrun) {
+        $request->add_state_variable( key => $_, value => $self->{'state_variables'}->{$_} )
+          for keys %{ $self->{'state_variables'} };
+        for (@actions) {
             $request->add_action(
                 moniker   => $_->moniker,
                 class     => $_->class,
                 order     => $_->order,
-                active    => $_->active,
+                active    => $_->active && (not $_->has_run),
+                has_run   => $_->has_run,
                 arguments => $_->arguments,
             );
         }
+        $request->continuation($self->request->continuation);
         my $cont = Jifty::Continuation->new(
             request  => $request,
             response => $self->response,
@@ -547,10 +580,9 @@ sub _redirect {
     $apache->header_out( Status => 302 );
     $apache->send_http_header();
 
-    # Abort or last_rule out of here
+    # Mason abort, or dispatcher abort out of here
     $self->mason->abort if $self->mason;
-    Jifty::Dispatcher::last_rule();
-
+    Jifty::Dispatcher::_abort;
 }
 
 =head3 save_continuation
@@ -672,7 +704,7 @@ sub tangent {
         @_
     );
     if ( defined wantarray ) {
-        return $clickable->generate->render;
+        return $clickable->generate;
     } else {
         $clickable->state_variable( $_ => $self->{'state_variables'}{$_} )
             for keys %{ $self->{'state_variables'} };
@@ -706,7 +738,7 @@ I<PARAMHASH>.
 
 sub link {
     my $self = shift;
-    return Jifty::Web::Form::Clickable->new(@_)->generate->render;
+    return Jifty::Web::Form::Clickable->new(@_)->generate;
 }
 
 =head3 return PARAMHASH
@@ -798,7 +830,7 @@ HTML-escapes the given string and returns it
 
 sub escape {
     my $self = shift;
-    return join '', map {Jifty::View::Mason::Handler::escape_utf8( \$_ ); $_} @_;
+    return join '', map {my $html = $_; Jifty::View::Mason::Handler::escape_utf8( \$html ); $html} @_;
 }
 
 =head3 escape_uri STRING
@@ -809,7 +841,7 @@ URI-escapes the given string and returns it
 
 sub escape_uri {
     my $self = shift;
-    return join '', map {Jifty::View::Mason::Handler::escape_uri( \$_ ); $_} @_;
+    return join '', map {my $uri = $_; Jifty::View::Mason::Handler::escape_uri( \$uri ); $uri} @_;
 }
 
 =head3 navigation
@@ -913,32 +945,23 @@ sub get_region {
     return $self->{'regions'}{$name};
 }
 
-=head3 region PARAMHASH, 
+=head3 region PARAMHASH
 
-Creates and renders a L<Jifty::Web::PageRegion>; the C<PARAMHASH> is
-passed directly to its L<Jifty::Web::PageRegion/new> method.  The
-region is then L<Jifty::Web::PageRegion/enter>ed, then
-L<Jifty::Web::PageRegion/render>ed, and finally
-L<Jifty::Web::PageRegion/exit>ed.
+The provided PARAMHASH is used to create and render a
+L<Jifty::Web::PageRegion>; the C<PARAMHASH> is passed directly to its
+L<Jifty::Web::PageRegion/new> method, and then
+L<Jifty::Web::PageRegion/render> is called.
 
 =cut
 
 sub region {
     my $self = shift;
 
-    # Add ourselves to the region stack
-    my $region = Jifty::Web::PageRegion->new(@_) or return;
-
-    # Enter the region
-    $region->enter;
+    # Create a region
+    my $region = Jifty::Web::PageRegion->new(@_) or return; 
 
     # Render it
-    $self->out( $region->render );
-
-    # Exit it when we're done
-    $region->exit;
-    
-    "";
+    $region->render;
 }
 
 =head3 current_region
@@ -955,76 +978,77 @@ sub current_region {
         : undef;
 }
 
-=head3 qualified_region
+=head3 qualified_region [REGION]
 
 Returns the fully qualified name of the current
-L<Jifty::Web::PageRegion>, or the empty string if there is none..
+L<Jifty::Web::PageRegion>, or the empty string if there is none.  If
+C<REGION> is supplied, gives the qualified name of C<REGION> were it
+placed in the current region.
 
 =cut
 
 sub qualified_region {
     my $self = shift;
-    return join( "-", map { $_->name } @{ $self->{'region_stack'} || [] } );
+    return join( "-", map { $_->name } @{ $self->{'region_stack'} || [] }, @_ );
 }
 
-=head3 serve_fragments
+=head3 include_css
 
-If the request is for individuals fragments, and not a full page, then
-this method fetches the requested fragments and serves them up,
-returning an XML document.
+Returns a C<< <link> >> tag for the compressed CSS
 
 =cut
 
-sub serve_fragments {
+sub include_css {
     my $self = shift;
-
-    my $output = "";
-    my $writer = XML::Writer->new( OUTPUT => \$output );
-    $writer->xmlDecl( "UTF-8", "yes" );
-    $writer->startTag("response");
-    for my $f ( $self->request->fragments ) {
-        # Set up the region stack
-        local Jifty->web->{'region_stack'} = [];
-        my @regions;
-        do {
-            push @regions, $f;
-        } while ($f = $f->parent);
-        
-        for $f (reverse @regions) {
-            my $new = $self->get_region( join '-', grep {$_} $self->qualified_region, $f->name );
-            $new ||= Jifty::Web::PageRegion->new(
-                name           => $f->name,
-                path           => $f->path,
-                region_wrapper => $f->wrapper,
-                parent         => Jifty->web->current_region,
-                defaults       => $f->arguments,
-            );
-            $new->enter;
-        }
-
-        # Stuff the rendered region into the XML
-        $writer->startTag( "fragment", id => Jifty->web->current_region->qualified_name );
-        $writer->cdata( Jifty->web->current_region->render );
-        $writer->endTag();
-
-        Jifty->web->current_region->exit while Jifty->web->current_region;
+    
+    if ( Jifty->config->framework('DevelMode') ) {
+        $self->out(
+            '<link rel="stylesheet" type="text/css" '
+            . 'href="/static/css/main.css" />'
+        );
     }
-    $writer->endTag();
+    else {
+        $self->generate_css;
+    
+        $self->out(
+            '<link rel="stylesheet" type="text/css" href="/__jifty/css/'
+            . __PACKAGE__->cached_css_digest . '.css" />'
+        );
+    }
+    
+    return '';
+}
 
-    # Spit out a correct content-type; we set this *here* instead of
-    # above because each of the subrequests attempts to set it to
-    # text/html -- so we have to override them after the fact.
-    $self->response->add_header("Content-Type" => 'text/xml; charset=utf-8');
+=head3 generate_css
 
-    # Print a header and the content, and then bail
-    my $apache = Jifty->handler->apache;
-    $apache->send_http_header();
+Checks if the compressed CSS is generated, and if it isn't, generates
+and caches it.
 
-    # Wide characters at this point should be harmlessly treated as UTF-8 octets.
-    no warnings 'utf8';
-    print $output;
+=cut
 
-    Jifty::Dispatcher::last_rule;
+sub generate_css {
+    my $self = shift;
+    
+    if (not defined __PACKAGE__->cached_css_digest
+            or Jifty->config->framework('DevelMode'))
+    {
+        Jifty->log->debug("Generating CSS...");
+        
+        my $css_dir = File::Spec->catdir(
+                           Jifty->config->framework('Web')->{'StaticRoot'},
+                           'css'
+                      );
+
+         my $css = CSS::Squish->concatenate(
+                      Jifty::Util->absolute_path(
+                          File::Spec->catpath( '', $css_dir, 'main.css' )
+                      )
+                   );
+
+         __PACKAGE__->cached_css( $css );
+        __PACKAGE__->cached_css_digest( md5_hex( $css ) );
+        __PACKAGE__->cached_css_mtime( time );
+    }
 }
 
 1;
